/*
 * The MIT License
 * Copyright © 2014-2021 Ilkka Seppälä
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.iluwatar.parameter.object;

import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static org.junit.jupiter.api.Assertions.assertEquals;

class ParameterObjectTest {

  private static final Logger LOGGER = LoggerFactory.getLogger(ParameterObjectTest.class);

  @Test
  public void testForDefaultSortBy() {
    //Creating parameter object with default value for SortBy set
    ParameterObject params = ParameterObject.newBuilder()
        .withType("sneakers")
        .sortOrder(SortOrder.DESC)
        .build();

    assertEquals(ParameterObject.DEFAULT_SORT_BY, params.getSortBy(),
        "Default SortBy is not set.");
    LOGGER.info("{} Default parameter value is set during object creation as no value is passed."
        , "SortBy");
  }

  @Test
  public void testForDefaultSortOrder() {
    //Creating parameter object with default value for SortOrder set
    ParameterObject params = ParameterObject.newBuilder()
        .withType("sneakers")
        .sortBy("brand")
        .build();

    assertEquals(ParameterObject.DEFAULT_SORT_ORDER, params.getSortOrder(),
        "Default SortOrder is not set.");
    LOGGER.info("{} Default parameter value is set during object creation as no value is passed."
        , "SortOrder");
  }
}
